// TestMca527.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"
#include "windows.h"
#include "time.h"
#include "conio.h"
#include "iomanip.h"
#include "iostream.h"

#define _FOR_COMPATIBILITY // COMM_INIT_ETHERNET is a function that is actually only available for compatibility reasons.

#include "mca_comm.h"

#define QS_FILLED           0x8000

#define QS_DATA_UNLOCK      2
#define QS_DATA_LOCK        3

#define MCA_SERIAL_NUMBER   1059  // The serial number of the used MCA527


void Offline()
{ COMM_CLOSE();

  cout<<"Unexpected communication error - press key to exit program."<<endl;

  while(!_kbhit());
}

int main(int argc, char* argv[])
{ unsigned long aulSpectrum[1024],ulSum,ulNumber=0;

  QUERY_SYSTEM_DATA QSystemData;
  QUERY_SPECTRA_EX2_LONG  QSpectra;

  if(!COMM_INIT_ETHERNET(MCA_SERIAL_NUMBER,1000,4,0)) return 0;

  cout<<"Measuring is initialized."<<endl;
  cout<<"(Press any key to exit the program.)"<<endl;

  if(MMCA_RESET()!=ERROR_OK
  || MMCA_SET_ADC_RES_DISCR(1024,0,1023)!=ERROR_OK
  || MMCA_SET_PRESET_REAL_TIME(1)!=ERROR_OK
  || MMCA_SET_REPEAT(0)!=ERROR_OK
  || MMCA_START_ACQUIRE(3,0)!=ERROR_OK
  //|| MMCA_SET_HIGH_VOLTAGES(800,0)!=ERROR_OK // Needed, if the high voltage is not automatically set.
  )
  { Offline();
    return 0;
  }

  while(1)
  { MMCA_QUERY_SYSTEM_DATA((char*)&QSystemData);

    if(QSystemData.unBufferState & QS_FILLED)
    { // If the buffer is filled, it can be read

      // In the example, 4 times 256 channels are read..
    
      MMCA_QUERY_SPECTRA_EX2(0,1,QS_DATA_LOCK,(char*)&QSpectra);        // At the first call, the buffer must be locked..

      memcpy(aulSpectrum,QSpectra.ulChannelContents,256*sizeof(unsigned long));

      MMCA_QUERY_SPECTRA_EX2(256,1,QS_DATA_LOCK,(char*)&QSpectra);

      memcpy(aulSpectrum+256,QSpectra.ulChannelContents,256*sizeof(unsigned long));

      MMCA_QUERY_SPECTRA_EX2(512,1,QS_DATA_LOCK,(char*)&QSpectra);

      memcpy(aulSpectrum+512,QSpectra.ulChannelContents,256*sizeof(unsigned long));

      MMCA_QUERY_SPECTRA_EX2(768,1,QS_DATA_UNLOCK,(char*)&QSpectra);    // At the last call, the buffer must be unlocked..

      memcpy(aulSpectrum+768,QSpectra.ulChannelContents,256*sizeof(unsigned long));

      ulSum=0;

      for(int i=0;i<1024;i++)
        ulSum+=aulSpectrum[i];

      cout<<"Measurement "<<setfill('0')<<setw(4)<<++ulNumber<<": Sum="<<ulSum<<endl;
    }
  
    Sleep(10);

    if(_kbhit()) break;
  }

  MMCA_STOP_ACQUIRE();
  
  COMM_CLOSE();
  
  return 0;
}
